/*--------------------------------------------------------------------------*/
/*       Copyright(C) 2004-2008 KONICA MINOLTA SENSING,INC.                 */
/*--------------------------------------------------------------------------*/
#include "StdAfx.h"
#include "Vivid9iControl.h"
#include <VISensorCreator.h>
#include <VICreator.h>
#include <Vivid9iPara.h>
#include <VividImportPara.h>
#include <VIVertexData.h>

#include <Vivid9iErrCode.h>
#include <VIParameter.h>
#include <Vivid9iCalibErr.h>
#include <Vivid9iCalibInfo.h>


#include <VISensorPara.h>
#include <Vivid910Para.h>

//---------------------------------------------------------------------------------------//
CVivid9iControl::CVivid9iControl(void)
{
	m_pSensor = NULL;
	m_pSensorPara = NULL;

}

//---------------------------------------------------------------------------------------//
CVivid9iControl::~CVivid9iControl(void)
{
    delete m_pSensor;

	delete m_pSensorPara;
}
//---------------------------------------------------------------------------------------//
void CVivid9iControl::SetErrCode(int nErrCode)
{
    m_nErrCode = nErrCode;
}

//---------------------------------------------------------------------------------------//
int CVivid9iControl::GetErrCode(void)
{
    return m_nErrCode;
}

//---------------------------------------------------------------------------------------//
//Active AF/AE & Scan
BOOL CVivid9iControl::DoScan(void)
{
	if( !m_pSensor || !m_pSensorPara ) return FALSE;

    //Get Sensor Setting
    BOOL bSuccess;
	
    bSuccess = m_pSensor->GetSensorPara(m_pSensorPara);
    if (!bSuccess)
    {
        int nErrCode;
        m_pSensor->GetErrCode(nErrCode);
        SetErrCode(nErrCode);
        return FALSE;
    }
    
#if 1

    //Set Sensor Setting
	switch( m_nVividType ) {
		case VI_VIVID9i_SENSOR:
			((CVivid9iPara*)m_pSensorPara)->m_bActAF = TRUE;
			((CVivid9iPara*)m_pSensorPara)->m_bActAE = TRUE;
			((CVivid9iPara*)m_pSensorPara)->m_bColor = TRUE;
			break;
		case VI_VIVID910_SENSOR:
			((CVivid910Para*)m_pSensorPara)->m_bActAF = TRUE;
			((CVivid910Para*)m_pSensorPara)->m_bActAE = TRUE;
			((CVivid910Para*)m_pSensorPara)->m_bColor = TRUE;
			((CVivid910Para*)m_pSensorPara)->m_nReleaseMode = 0; // 0:FINE 1:FAST
			break;
		default:
			return FALSE;
			break;
	}

    bSuccess = m_pSensor->SetSensorPara(m_pSensorPara);
    if (!bSuccess)
    {
        int nErrCode;
        m_pSensor->GetErrCode(nErrCode);
        SetErrCode(nErrCode);
        return FALSE;
    }

    //Active AF/AE
    bSuccess = m_pSensor->Ready();
    if (!bSuccess)
    {
        int nErrCode;
        m_pSensor->GetErrCode(nErrCode);
        SetErrCode(nErrCode);
        return FALSE;
    }

#else // 041001 manual setting

    m_p9iPara->m_bActAE = FALSE;
    m_p9iPara->m_bActAF = FALSE;
    m_p9iPara->m_bColor = TRUE;
    m_p9iPara->m_wDistance = 794;
    m_p9iPara->m_byLdPower = 50;
    m_p9iPara->m_byGain = 0;
    m_p9iPara->m_byScanNum = 1; // Number of the scans per one shot (1-4)

    bSuccess = m_pSensor->SetSensorPara(m_p9iPara);
    if (!bSuccess)
    {
        int nErrCode;
        m_pSensor->GetErrCode(nErrCode);
        SetErrCode(nErrCode);
        return FALSE;
    }
#endif

    //Scan
    bSuccess = m_pSensor->Measure();
    if (!bSuccess)
    {
        int nErrCode;
        m_pSensor->GetErrCode(nErrCode);
        SetErrCode(nErrCode);
        return FALSE;
    }

    return TRUE;
}


//---------------------------------------------------------------------------------------//
//Save CDK file
BOOL CVivid9iControl::SaveRawData(void)
{
	if( !m_pSensor || !m_pSensorPara ) return FALSE;

	CString filename;

	if( m_nVividType == VI_VIVID9i_SENSOR ) {
		CFileDialog fileDlg(FALSE, _T("cdk"),NULL,OFN_FILEMUSTEXIST,_T("Camera Data Files (*.cdk)|*.cdk||"), NULL);
		if( fileDlg.DoModal() != IDOK ) return TRUE;
		filename = fileDlg.GetPathName();
	}
	else if( m_nVividType == VI_VIVID910_SENSOR ) {
		CFileDialog fileDlg(FALSE, _T("cdm"),NULL,OFN_FILEMUSTEXIST,_T("Camera Data Files (*.cdm)|*.cdm||"), NULL);
		if( fileDlg.DoModal() != IDOK ) return TRUE;
		filename = fileDlg.GetPathName();
	}

    BOOL bSuccess; 
    bSuccess = m_pSensor->SaveRawData(filename);
    if (!bSuccess)
    {
        int nErrCode;
        m_pSensor->GetErrCode(nErrCode);
        SetErrCode(nErrCode);
        return FALSE;
    }
    return bSuccess;
}

//---------------------------------------------------------------------------------------//
//Save vvd file
BOOL CVivid9iControl::SavePolygonData(void)
{
	if( !m_pSensor || !m_pSensorPara ) return FALSE;

	CFileDialog fileDlg(FALSE,_T("vvd"),NULL,OFN_FILEMUSTEXIST,_T("Polygon Data Files (*.vvd)|*.vvd||"), NULL);
    if( fileDlg.DoModal() != IDOK ) return TRUE;
    CString filename    = fileDlg.GetPathName();

	TCHAR drive[_MAX_DRIVE], dir[_MAX_DIR], fname[_MAX_FNAME],ext[_MAX_EXT];
	_tsplitpath_s( (LPCTSTR)filename, drive, dir, fname, ext );

	// Setting the import parameter
    CVividImportPara VIVIDImportPara;
    VIVIDImportPara.m_nReduce   = VIReduceNone	            ;
    VIVIDImportPara.m_nFilter   = VINoiseFilterHighQuality	;
    VIVIDImportPara.m_nCutData  = VINoneCut		            ;
    VIVIDImportPara.m_nFillHole = VIFillOff		            ;
    
    //Getting Polygon Data;
    CVIObject Polygon;
    BOOL bSuccess; 
    bSuccess = m_pSensor->GetPolygonData(Polygon,&VIVIDImportPara,fname);
    if (!bSuccess)
    {
        int nErrCode;
        m_pSensor->GetErrCode(nErrCode);
        SetErrCode(nErrCode);
        return FALSE;
    }

    //Save Polygon Data
    CVISensorPolygonFileProcess* pPolygonFileProcess;
    pPolygonFileProcess = VISensorPolygonFileProcessCreate(VI_VIVID9i_SENSOR);
    bSuccess = pPolygonFileProcess->SavePolygonData(Polygon,filename);
    if (!bSuccess)
    {
        int nErrCode;
        pPolygonFileProcess->GetErrCode(nErrCode);
        SetErrCode(nErrCode);
        return FALSE;
    }
    
    VISensorPolygonFileProcessDelete(pPolygonFileProcess);
	pPolygonFileProcess = NULL;
    
    return TRUE;
}


//---------------------------------------------------------------------------------------//
//Save ASCII file
BOOL CVivid9iControl::SaveASCIIData(void)
{
	if( !m_pSensor || !m_pSensorPara ) return FALSE;

	// Setting the import parameter
    CVividImportPara VIVIDImportPara;
    
    // 041001
    VIVIDImportPara.m_nReduce   = VIReduceNone;
    VIVIDImportPara.m_nFilter   = VINoiseFilter;
    VIVIDImportPara.m_nCutData  = VIDeg5Cut;
    VIVIDImportPara.m_nFillHole = VIFillOff;

    //Getting Vertex
    BOOL bSuccess; 
    CVIVertexData   VertexData;
    bSuccess = m_pSensor->GetVertexData(&VertexData,&VIVIDImportPara);
    if (!bSuccess)
    {
        int nErrCode;
        m_pSensor->GetErrCode(nErrCode);
        SetErrCode(nErrCode);
        return FALSE;
    }
    
#if 1
    CFileDialog fileDlg(FALSE, _T("asc"),NULL,OFN_FILEMUSTEXIST,_T("ASCII Files (*.asc)|*.asc||"), NULL);
    if( fileDlg.DoModal() != IDOK ) return TRUE;
    CString filename    = fileDlg.GetPathName();
#else // 041001 auto file saveing
    CString filename;
    filename.Format("%s-%d.asc","Data",1);;
#endif

	//File export
	CFile fl;
	if (!fl.Open( filename,	CFile::modeCreate | CFile::modeWrite))
	{
		SetErrCode(VI_ERR_FILE_WRITE);
        return FALSE;
    }
    CArchive ar(&fl, CArchive::store);

    //Setting Header
    CString strHeader;

    int width,height;
    VertexData.GetSize(width,height);
#if 1
    strHeader.Format( _T("# vivid 3D Data : \nwidth %d\nheight %d\n"),width,height);
#endif // 041001 data information cancel 

    ar.WriteString( strHeader );
    // Write Vertex Data
    float x, y, z;
    CString strLine;
    for ( int i = 0;i < height; i++)
    {
        for ( int j = 0;j < width; j++)
        {
            x = (float)VertexData.m_pVertex[i * width + j].point.x;
            y = (float)VertexData.m_pVertex[i * width + j].point.y;
            z = (float)VertexData.m_pVertex[i * width + j].point.z;
            if ((x == VI_INVALID_DATA_3D) || (y == VI_INVALID_DATA_3D)||(y == VI_INVALID_DATA_3D))  continue;
            strLine.Format( _T("%.3f %.3f %.3f\r\n"), x, y, z);
            ar.WriteString( strLine );
        }
    }
    //Setting footer
    CString strFooter;
    strFooter = _T("**end cloud**\n");
    ar.WriteString( strFooter );

    ar.Close();
	fl.Close();

    return TRUE;
}

//---------------------------------------------------------------------------------------//
//Get Monitor Image
BOOL CVivid9iControl::GetMonitorData(CVvdImage& ImageData,const BOOL bLog,const BOOL bDark)
{
	if( !m_pSensor || !m_pSensorPara ) return FALSE;

	CVividImportPara VIVIDImportPara;
    VIVIDImportPara.m_bDark = bDark ;
    VIVIDImportPara.m_bLog  = bLog  ;
    BOOL bSuccess;
    bSuccess = m_pSensor->GetVvdImageData(ImageData,VIVID_MONITOR_DATA,&VIVIDImportPara);
    if (!bSuccess)
    {
        int nErrCode;
        m_pSensor->GetErrCode(nErrCode);
        SetErrCode(nErrCode);
        return FALSE;
    }
    return bSuccess;
}

//---------------------------------------------------------------------------------------//
//Get Color Image
BOOL CVivid9iControl::GetColorData(CVvdImage& ImageData,const BOOL bLog,const BOOL bDark)
{
	if( !m_pSensor || !m_pSensorPara ) return FALSE;

	CVividImportPara VIVIDImportPara;
    VIVIDImportPara.m_bDark = bDark ;
    VIVIDImportPara.m_bLog  = bLog  ;
    BOOL bSuccess;
    bSuccess = m_pSensor->GetVvdImageData(ImageData,VIVID_COLOR_DATA,&VIVIDImportPara);
    if (!bSuccess)
    {
        int nErrCode;
        m_pSensor->GetErrCode(nErrCode);
        SetErrCode(nErrCode);
        return FALSE;
    }
    return bSuccess;
}

//---------------------------------------------------------------------------------------//
//Field Calibration 
BOOL CVivid9iControl::DoCalibration(void)
{
	if( !m_pSensor || !m_pSensorPara ) return FALSE;

	BOOL bSuccess;
	char str[64];
    CString strMes;
	CVivid9iCalibInfo	CalibInfo;
    CVivid9iCalibErr    CalibErr;

	m_pSensor->GetCalibrationInfo(&CalibInfo);

	time_t calibdate = (time_t)CalibInfo.m_lCalibrationDate;
	struct tm t_st;
	gmtime_s( &t_st, &calibdate );
	_tcsftime(str, sizeof(str), _T("%b. %d %Y"), &t_st);
    strMes.Format( _T("Do calibration?\r\n\r\nLast calibration date : %s"), str  );


    if ( AfxMessageBox( strMes, MB_OKCANCEL | MB_ICONQUESTION ) == IDCANCEL ) return TRUE;

    bSuccess = m_pSensor->Calibrate(&CalibErr);
    if (!bSuccess)
    {
        int nErrCode;
        m_pSensor->GetErrCode(nErrCode);
        SetErrCode(nErrCode);
        return FALSE;
    }
    strMes.Format( _T("Field Calibration was completed.\r\nResults:\r\n X1=%.3f\tY1=%.3f\r\n X2=%.3f\tY2=%.3f"),
                    CalibErr.m_fX1ErrLength,  CalibErr.m_fY1ErrLength,
                    CalibErr.m_fX2ErrLength,  CalibErr.m_fY2ErrLength  );

	AfxMessageBox( strMes, MB_OK | MB_ICONINFORMATION );

    return bSuccess;
}

BOOL CVivid9iControl::SetVividType(int nType)
{
	VISensorDelete(m_pSensor);
	delete m_pSensorPara;
	m_pSensor = NULL;
	m_pSensorPara = NULL;
	m_nVividType = -1;

	switch( nType ) {
		case VI_VIVID9i_SENSOR:
			m_pSensor   = VISensorCreate(VI_VIVID9i_SENSOR);
			m_pSensorPara = new CVivid9iPara;
			break;
		case VI_VIVID910_SENSOR:
			m_pSensor   = VISensorCreate(VI_VIVID910_SENSOR);
			m_pSensorPara = new CVivid910Para;
			break;
		default:
			return FALSE;
			break;
	}

	m_nVividType = nType;


    m_nErrCode  = VI_ERR_NONE;
    BOOL bSuccess = m_pSensor->Init();
    if (!bSuccess)
    {
        int nErrCode;
        m_pSensor->GetErrCode(nErrCode);
        SetErrCode(nErrCode);
    }
	return bSuccess;
}