////////////////////////////////////////////////////////////
//
//		Filename:		CVIVector.h
//		Version:		0.1
//		Author:			Zhu Huimin
//		History:		Ver 0.0		(1996/8/30)
//						Ver 0.1		(1996/9/5)
//						Ver 0.2		(2000/1/20)
//
//		Copyright:		This source was designed by Zhu Huimin in 1996/9/5.
//						To use this source code free, don't remove the author's 
//							name & comment, thank you very much!
//						If you have good idea, please send email to: zhm6909@guomai.sh.cn
//
//		Description:	
//			CVIVector_2:	+,  -,  *,  /,  * t,  / t,  * m22
//						+=, -=, *=, /=, *= t, /= t, *= m22
//						==, !=
//						<<, >>
//						GetX, GetY
//			CVIVector_3:	+,  -,  *,  /,  * t,  / t,  * m33
//						+=, -=, *=, /=, *= t, /= t, *= m33
//						==, !=
//						<<, >>
//						GetX, GetY, GetZ
//			CVIVector_4:	+,  -,  *,  /,  * t,  / t,  * m44
//						+=, -=, *=, /=, *= t, /= t, *= m44
//						==, !=
//						<<, >>
//						GetX, GetY, GetZ, GetH
//
//		Comment:
//
////////////////////////////////////////////////////////////
/*--------------------------------------------------------------------------*/
/*       Copyright(C) 2004-2008 KONICA MINOLTA SENSING,INC.                 */
/*--------------------------------------------------------------------------*/

#ifndef	__CVIVECTOR_H
#define	__CVIVECTOR_H


// ---------------- 2-Dimension -----------------
template< class T >
class CVIVector_2
{
public:
	T x,y;
public:
	CVIVector_2<T>(){};			// no any initialize operation
	CVIVector_2<T>( T x1, T y1 ) : x(x1), y(y1) {};
	CVIVector_2<T>( const CVIVector_2<T>& v ) : x(v.x), y(v.y){};
public:
	const CVIVector_2<T> operator + ( const CVIVector_2<T>& v ) const
		{ return CVIVector_2<T>( x+v.x, y+v.y ); };
	const CVIVector_2<T> operator - ( const CVIVector_2<T>& v ) const
		{ return CVIVector_2<T>( x-v.x, y-v.y ); };
	const CVIVector_2<T> operator * ( const CVIVector_2<T>& v ) const
		{ return CVIVector_2<T>( x*v.x, y*v.y ); };
	const CVIVector_2<T> operator / ( const CVIVector_2<T>& v ) const
		{ return CVIVector_2<T>( x/v.x, y/v.y ); };
	const CVIVector_2<T> operator * ( const T& t ) const
		{ return CVIVector_2<T>( x*t, y*t ); };
	const CVIVector_2<T> operator / ( const T& t ) const
		{ return CVIVector_2<T>( x/t, y/t ); };
	const CVIVector_2<T> operator += ( const CVIVector_2<T>& v )
		{ x+=v.x; y+=v.y; return *this; };
	const CVIVector_2<T> operator -= ( const CVIVector_2<T>& v )
		{ x-=v.x; y-=v.y; return *this; };
	const CVIVector_2<T> operator *= ( const CVIVector_2<T>& v )
		{ x*=v.x; y*=v.y; return *this; };
	const CVIVector_2<T> operator /= ( const CVIVector_2<T>& v )
		{ x/=v.x; y/=v.y; return *this; };
	const CVIVector_2<T> operator *= ( const T& t )
		{ x*=t; y*=t; return *this; };
	const CVIVector_2<T> operator /= ( const T& t )
		{ x/=t; y/=t; return *this; };

	// accessing function
	T&	GetX(){ return x; };
	T&	GetY(){ return y; };
};

template< class T >
inline int operator == ( const CVIVector_2<T>& v1, const CVIVector_2<T>& v2 )
{ return ( v1.x == v2.x ) && ( v1.y == v2.y ); };

template< class T >
inline int operator != ( const CVIVector_2<T>& v1, const CVIVector_2<T>& v2 )
{ return !( v1 == v2 ); };


// ---------------- 3-Dimension -----------------
template< class T >
class CVIVector_3
{
public:
	T x,y,z;
public:
	CVIVector_3<T>(){};			// no any initialize operation
	CVIVector_3<T>( T x1, T y1, T z1 ) : x(x1), y(y1), z(z1) {};
	CVIVector_3<T>( const CVIVector_3<T>& v ) : x(v.x), y(v.y), z(v.z) {};
public:
	const CVIVector_3<T> operator + ( const CVIVector_3<T>& v ) const
		{ return CVIVector_3<T>( x+v.x, y+v.y, z+v.z ); };
	const CVIVector_3<T> operator - ( const CVIVector_3<T>& v ) const
		{ return CVIVector_3<T>( x-v.x, y-v.y, z-v.z ); };
	const CVIVector_3<T> operator * ( const CVIVector_3<T>& v ) const
		{ return CVIVector_3<T>( x*v.x, y*v.y, z*v.z ); };
	const CVIVector_3<T> operator / ( const CVIVector_3<T>& v ) const
		{ return CVIVector_3<T>( x/v.x, y/v.y, z/v.z ); };
	const CVIVector_3<T> operator * ( const T& t ) const
		{ return CVIVector_3<T>( x*t, y*t, z*t ); };
	const CVIVector_3<T> operator / ( const T& t ) const
		{ return CVIVector_3<T>( x/t, y/t, z/t ); };
	const CVIVector_3<T> operator += ( const CVIVector_3<T>& v )
		{ x+=v.x; y+=v.y; z+=v.z; return *this; };
	const CVIVector_3<T> operator -= ( const CVIVector_3<T>& v )
		{ x-=v.x; y-=v.y; z-=v.z; return *this; };
	const CVIVector_3<T> operator *= ( const CVIVector_3<T>& v )
		{ x*=v.x; y*=v.y; z*=v.z; return *this; };
	const CVIVector_3<T> operator /= ( const CVIVector_3<T>& v )
		{ x/=v.x; y/=v.y; z/=v.z; return *this; };
	const CVIVector_3<T> operator *= ( const T& t )
		{ x*=t; y*=t; z*=t; return *this; };
	const CVIVector_3<T> operator /= ( const T& t )
		{ x/=t; y/=t; z/=t; return *this; };
	// accessing function
	T&	GetX(){ return x; };
	T&	GetY(){ return y; };
	T&	GetZ(){ return z; };
};

// if you want to use != operator, you must include stl\function.h
template< class T >
inline int operator == ( const CVIVector_3<T>& v1, const CVIVector_3<T>& v2 )
{ return ( v1.x == v2.x ) && ( v1.y == v2.y ) && ( v1.z == v2.z ); };

template< class T >
inline int operator != ( const CVIVector_3<T>& v1, const CVIVector_3<T>& v2 )
{ return !( v1 == v2 ); };


// ---------------- 4-Dimension -----------------
template< class T >
class CVIVector_4
{
public:
	T x,y,z,h;
public:
	CVIVector_4<T>(){};			// no any initialize operation
	CVIVector_4<T>( T x1, T y1, T z1, T h1 ) : x(x1), y(y1), z(z1), h(h1) {};
	CVIVector_4<T>( const CVIVector_4<T>& v ) : x(v.x), y(v.y), z(v.z), h(v.h) {};
public:
	const CVIVector_4<T> operator + ( const CVIVector_4<T>& v ) const
		{ return CVIVector_4<T>( x+v.x, y+v.y, z+v.z, h+v.h ); };
	const CVIVector_4<T> operator - ( const CVIVector_4<T>& v ) const
		{ return CVIVector_4<T>( x-v.x, y-v.y, z-v.z, h-v.h ); };
	const CVIVector_4<T> operator * ( const CVIVector_4<T>& v ) const
		{ return CVIVector_4<T>( x*v.x, y*v.y, z*v.z, h*v.h ); };
	const CVIVector_4<T> operator / ( const CVIVector_4<T>& v ) const
		{ return CVIVector_4<T>( x/v.x, y/v.y, z/v.z, h/v.h ); };
	const CVIVector_4<T> operator * ( const T& t ) const
		{ return CVIVector_4<T>( x*t, y*t, z*t, h*t ); };
	const CVIVector_4<T> operator / ( const T& t ) const
		{ return CVIVector_4<T>( x/t, y/t, z/t, h/t ); };
	const CVIVector_4<T> operator += ( const CVIVector_4<T>& v )
		{ x+=v.x; y+=v.y; z+=v.z; h+=v.h; return *this; };
	const CVIVector_4<T> operator -= ( const CVIVector_4<T>& v )
		{ x-=v.x; y-=v.y; z-=v.z; h-=v.h; return *this; };
	const CVIVector_4<T> operator *= ( const CVIVector_4<T>& v )
		{ x*=v.x; y*=v.y; z*=v.z; h*=v.h; return *this; };
	const CVIVector_4<T> operator /= ( const CVIVector_4<T>& v )
		{ x/=v.x; y/=v.y; z/=v.z; h/=v.h; return *this; };
	const CVIVector_4<T> operator *= ( const T& t )
		{ x*=t; y*=t; z*=t; h*=t; return *this; };
	const CVIVector_4<T> operator /= ( const T& t )
		{ x/=t; y/=t; z/=t; h/=t; return *this; };
	// accessing function
	T&	GetX(){ return x; };
	T&	GetY(){ return y; };
	T&	GetZ(){ return z; };
	T&	GetH(){ return h; };
};

// if you want to use != operator, you must include stl\function.h
template< class T >
inline int operator == ( const CVIVector_4<T>& v1, const CVIVector_4<T>& v2 )
{ return ( v1.x == v2.x ) && ( v1.y == v2.y ) && ( v1.z == v2.z ) && ( v1.h == v2.h ); };

template< class T >
inline int operator != ( const CVIVector_4<T>& v1, const CVIVector_4<T>& v2 )
{ return !( v1 == v2 ); };

#endif	//__CVECTOR_H
