/*--------------------------------------------------------------------------*/
/*       Copyright(C) 2004-2008 KONICA MINOLTA SENSING,INC.                 */
/*--------------------------------------------------------------------------*/
// VividObject.h: interface of the CVividObject class
//
//////////////////////////////////////////////////////////////////////

#if !defined(__VIVID_OBJECT_H__)
#define __VIVID_OBJECT_H__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <vividIItypes.h>
#include <vvdimage.h>
#ifdef DATAPROCESS_EXPORTS
#define DATAPROCESS_API __declspec(dllexport)
#else
#define DATAPROCESS_API __declspec(dllimport)
#endif

// Attribute for Image Node
#define		VI_IMAGE_ATTR_SW		0x00000001  
#define		VI_CAMERA_ATTR_SW		0x00000001

typedef struct {
    VVD_Char	product[16];
    VVD_Char	version[16];
    VVD_Char	name[32];
    long		numChildren;
    long		type;
    long		imageOffset;
    long		numImages;
    VVD_Char	dumb[4];
    long		magic;
    VVD_Char	dummy[40];
} Vivid3HeaderFormat;

#define	    VIVID_MODEL_MESH		1
#define	    VIVID_MODEL_LINEAR		2
#define	    VIVID_MODEL_BEZIER		3
#define	    VIVID_MODEL_CHARLINE	4
#define		VIVID_MODEL_ELEMENT		5
#define	    VIVID_SCENE_FILE		0

#define	    VIVID_HEADER_VERSION   "1.10"
#define	    VIVID_HEADER_VERSION3  "3.01"
#define	    VIVID_HEADER_PRODUCT   "VIVID"

#define	    VIVID_HEADER_MAGIC_NUMBER  0xabcdef01

#define	    VIVID_HEADER_ILLEGAL_FORMAT	-1

//---------------------------------------------------------------------------------------//
class  DATAPROCESS_API CVITextureCoordNode
{
public:
    CVITextureCoordNode();
    virtual ~CVITextureCoordNode();
    CVITextureCoordNode(const CVITextureCoordNode & sc);
    CVITextureCoordNode& operator = ( const CVITextureCoordNode & sc );
    double   m_dTexCoordU;
    double   m_dTexCoordV;
protected:
    void Copy(const CVITextureCoordNode &sc);

};

//---------------------------------------------------------------------------------------//
class  DATAPROCESS_API CVITextureCoord
{
public:
    CVITextureCoord();
    virtual ~CVITextureCoord();
    CVITextureCoord(const CVITextureCoord & sc);
	BOOL SetColum(int colum);
    CVITextureCoord& operator = (CVITextureCoord & sc );
    CVITextureCoordNode*        m_pTexCoordNodes    ;   // TextureCoord Node Array
    VVD_Int                     m_numTexCoordNodes  ;   // the number of nodes
protected:
    void Copy(const CVITextureCoordNode &sc);

};

/*-------------------------------------------------------------------------*/

class DATAPROCESS_API CVITextureCoordList
{
public:
	BOOL Alloc(int size);
	BOOL SetTextureCoord(int TexCoordNumber,CVITextureCoordNode& coord1,CVITextureCoordNode& coord2,CVITextureCoordNode& coord3);
	BOOL SetTextureCoord(int TexCoordNumber,CVITextureCoordNode& coord1,CVITextureCoordNode& coord2,
                                                                    CVITextureCoordNode& coord3,CVITextureCoordNode& coord4);
	BOOL Append(int size);
	void Delete(int index);
    CVITextureCoordList();
    virtual ~CVITextureCoordList();
    CVITextureCoord*    m_pTextureCoord     ;		// texturecoord array
    VVD_Int			    m_numTextureCoord   ;	    // the number of texturecoord
protected:
	int m_nPageSize;
	int m_nAllocSize;
    int m_nImageNum;
public:
    virtual void SetImageNum(int nImageNum);
    int ImageNum(void);
};

//---------------------------------------------------------------------------------------//
class DATAPROCESS_API CVITextureIndexList
{
public:
	BOOL Alloc(int size);
	BOOL SetTextureIndex(int TexCoordNumber,DWORD nIndex);
	BOOL Append(int size);
	void Delete(int index);
    CVITextureIndexList();
    virtual ~CVITextureIndexList();
    DWORD*      m_pTextureIndex     ;		// textureindex array
    VVD_Int     m_numTextureIndex   ;	    // the number of textureindex
protected:
	int m_nPageSize;
	int m_nAllocSize;
    int m_nImageNum;
public:
    virtual void SetImageNum(int nImageNum);
    int ImageNum(void);
};

/* 3-D point */
typedef struct {    
    VVD_Point3	position;   		/* 3-D coordinates */
    VVD_Flag    selectFlag;			/* vertex selection flag(not used) */
} VVD_Vertex;

/*
typedef struct {
	VVDII_Point3	position;		// 3-D coordinates 
} VVDII_Vertex;
*/
class VVDII_Vertex{
public:
	VVDII_Point3	position;		// 3-D coordinates 
};
/* polygon node index */
typedef struct {
    VVD_Int   vertex;  	    		/* vertex index */
    VVD_Vector3  normal;			/* normal vector at the node */
} VVD_PolyNode;

/*-------------------------------------------------------------------------*/
class  CVIPolyNode
{
public:
    CVIPolyNode();
    virtual ~CVIPolyNode();
    CVIPolyNode(const CVIPolyNode & sc);
    CVIPolyNode& operator = ( const CVIPolyNode & sc );
    long   m_vertex;  	    	    /* vertex index */
};

/*-------------------------------------------------------------------------*/
class DATAPROCESS_API CVIPolygon
{
public:
	BOOL SetColum(int colum);
    CVIPolygon();
    CVIPolygon(const CVIPolygon & sc);
    virtual ~CVIPolygon();
    CVIPolygon& operator = ( CVIPolygon & sc );
    CVIPolyNode	*m_polyNodes; 	/* polygon node array */
    VVD_Int			m_numPolyNodes;	/* the number of nodes */
};

/*-------------------------------------------------------------------------*/
class DATAPROCESS_API CVIMesh
{
public:
	BOOL Alloc(int size);
	BOOL SetPoly(int PolyNumber,int index1,int index2,int index3 , int index4 = -1);
	BOOL Append(int size);
	void Delete(int index);
    CVIMesh();
    virtual ~CVIMesh();
    CVIPolygon    *m_polygons;		/* polygons array */
    VVD_Int			m_numPolygons;	/* the number of polygons */
    VVD_Flag	    m_normalFlag;	/* flag whether normal had calculated(not used) */
    CVITextureCoordList             m_TextureCoord; // Texture Coord
    CVITextureIndexList             m_TextureIndex; // Texture Index
    VICameraInfo                    m_CameraInfo;
private:
	int m_nPageSize;
	int m_nAllocSize;
public:
    void SetImageNum(int nImageNum);
    int ImageNum(void);
    BOOL SetTextureCoord(int nTextureIndex, int nTexCoordNumber, CVITextureCoordNode& coord1, CVITextureCoordNode& coord2, CVITextureCoordNode& coord3, CVITextureCoordNode& coord4);
    BOOL SetTextureCoord(int nTextureIndex, int nTexCoordNumber, CVITextureCoordNode& coord1, CVITextureCoordNode& coord2, CVITextureCoordNode& coord3);
};

typedef enum {
    VVDII_MESH,
    VVDII_LINE
} VVDII_ModelType;

/* types of line data */
typedef enum {
    VVD_LINEAR,
    VVD_BEZIER,
    VVD_CHARLINE
} VVD_LineType;

typedef struct {
    VVD_Float	x, y, z;
    VVD_Flag 	selectFlag;
}VVD_LinePoint;

/* linear line */
typedef struct {
    VVD_LinePoint	*vertices;
    VVD_Int	    	numVertices;
} VVD_LinearLine;

/* continuity of bezier line */
typedef enum {
    VVD_BEZIER_C0,
    VVD_BEZIER_C1,
    VVD_BEZIER_G1
} VVD_BezierConstraint;

/* bezier line */
typedef struct vvd_bezierNode{
    VVD_LinePoint 	cp, front, back;
    VVD_BezierConstraint constraint;
    struct vvd_bezierNode *next;
} VVD_BezierNode;

typedef struct {
    VVD_BezierNode  *nodes;
    VVD_Int  	    numBezierNodes;
    VVD_Flag 	    closeFlag;
}VVD_BezierLine;

typedef enum {
    VVD_CYLINDER,
    VVD_PARTIAL_CYLNDR,
    VVD_PLANE
} VVD_CharLineType;

/* character line */
typedef struct{
    VVD_Int		id;
    VVD_BezierLine		**pline;    /* latitudinal bezier lines */
    VVD_Int	    		numPline;   /* number of latitudinal bezier lines */
    VVD_BezierLine		**qline;    /* logitudinal bezier lines  */
    VVD_Int	    		numQline;   /* number of longitudinal bezier lines */
    VVD_CharLineType	type;	    /* type of character line */
    VVD_Point3			axisTop;    /* coordinates of top point of axis */
    VVD_Point3			axisBottom; /* coordinates of bottom point of axis */
    VVD_Vector3			axisVector; /* unit vector of axis(from bottom to top) */
    VVD_Vector3			projVector; /* projection vector*/
    VVD_Flag			closeFlag;  /* flag whether close top and bottom*/
    VVD_Float			topPos;	    
    VVD_Float			bottomPos;  
    VVD_Vector3			baseVector; 
    VVD_Int				targetID;  
    VVD_Flag			doneInit;   
} VVD_CharLine;

typedef union{
    VVD_LinearLine		*linear;
    VVD_BezierLine		*bezier;
    VVD_CharLine		*charline;
}  VVD_LineData;

typedef struct{
    VVD_LineType		type;	    	/* line type  */
    VVD_LineData    	linedata;  	    /* line data  */
} VVDII_Line;

/*-------------------------------------------------------------------------*/
class DATAPROCESS_API CVIModelData
{
public:
    CVIModelData();
    virtual ~CVIModelData();
    union
    {
        VVDII_Line	*m_line;
        CVIMesh 	*m_mesh;
    };
};

/*-------------------------------------------------------------------------*/
#define	    VVD_ELEMENT_READ	    0
#define	    VVD_ELEMENT_CREATED	    1
#define	    VVD_ELEMENT_MODIFIED	2

class DATAPROCESS_API CVIModel
{
public:
    CVIModel();
    virtual ~CVIModel();
	void AllocVertices(int num);
	void AllocPolygons(int num);
	void AllocMesh();
	void ClearVertices();

    VVDII_ModelType   m_type;			/* model type */
    CVIModelData   m_modeldata;   	    /* model data */
    VVDII_Vertex	*m_vertices;	    /* vertex array */
    VVD_Int			m_numVertices;	    /* the number of vertices */
};

/*-------------------------------------------------------------------------*/
#define ELEMENTNAME_MAXLENGTH   33

class DATAPROCESS_API CVIObject 
{
public:
    CVIObject();
    virtual ~CVIObject();
    VVD_Char	*m_name;		/* element name (not used) */
    VVD_Int		m_id;			/* element ID (not used) */
    CVIModel	*m_models;		/* model data array */
    VVD_Int  	m_numModels;	/* th number of models */
    CVIImage	*m_images;		/* color image array */
    VVD_Int		m_numImages;	/* the number of color images */
    VVD_Flag	m_showFlag;	
    VVD_Flag	m_modified;	
};

class DATAPROCESS_API CVIObjectList
{
public:
	BOOL Init(int ObjectNum);
    CVIObjectList();
    virtual ~CVIObjectList();
    CVIObject	**m_objects;
    VVD_Int  	m_numObjects;
};

#endif
